package ch.rakudave.jnetmap.util;

import java.awt.image.BufferedImage;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Hashtable;

import javax.imageio.ImageIO;
import javax.swing.ImageIcon;

import ch.rakudave.jnetmap.util.logging.Logger;

/**
 * @author rakudave
 */
public class Icons {
	private static Hashtable<String, ImageIcon> icons;
	private static Hashtable<String, ImageIcon> deviceIcons;
	private static Hashtable<String, ImageIcon> customCache = new Hashtable<String, ImageIcon>();
	public static final File iconDir = new File(IO.userDir, "icons");
	public static final File devicesDir = new File(IO.userDir, "devices");
	private static String currentIconTheme = Settings.get("icon.theme", "Elementary");
	private static String currentDeviceTheme = Settings.get("device.theme", "cisco");
	
	public static String currentTheme() {
		return currentIconTheme;
	}
	
	public static String currentDeviceTheme() {
		return currentDeviceTheme;
	}

	public static ImageIcon get(String id) {
		if (icons == null) load();
		ImageIcon icon = icons.get(id);
		if (icon == null) {
			Logger.warn("No icon named " + id + " in theme " + currentIconTheme);
			icon = icons.get("jnetmap_small");
			if (icon == null) icon = new ImageIcon();
		}
		return icon;
	}
	
	public static ImageIcon getCisco(String id) {
		if (deviceIcons == null) load();
		ImageIcon icon = deviceIcons.get(id);
		if (icon == null) {
			Logger.warn("No icon named " + id + " from cisco");
			icon = deviceIcons.get("workstation");
			if (icon == null) icon = new ImageIcon();
		}
		return icon;
	}

	public static String[] getThemes() {
		String[] files = iconDir.list();
		Arrays.sort(files);
		return files;
	}

	public static String[] getDeviceThemes() {
		String[] files = devicesDir.list();
		Arrays.sort(files);
		return files;
	}
	
	public static boolean importTheme(File zip) {
		return IO.unzip(zip, iconDir);
	}
	
	public static boolean importDeviceTheme(File zip) {
		return IO.unzip(zip, devicesDir);
	}
		
	public static void load() {
		try {
			loadIcons(currentIconTheme);
			loadDevices(currentDeviceTheme);
		} catch (Exception e) {
			loadIcons("Elementary");
			loadDevices("cisco");
		}
	}
	
	public static synchronized void loadIcons(String theme) {
			File themeDir = new File(iconDir, theme);
		try {
			Logger.info("Loading icon-theme: " + theme);
			icons = new Hashtable<String, ImageIcon>();
			for (File f : themeDir.listFiles()) {
				icons.put(f.getName().replace(".png", ""), new ImageIcon(f.getAbsolutePath()));
			}
			currentIconTheme = theme;
			Settings.put("icon.theme", theme);
		} catch (Exception e) {
			Logger.fatal("Unable to load icons from "+themeDir.getAbsolutePath(), e);
		}
	}
	
	public static synchronized void loadDevices(String theme) {
		File themeDir = new File(devicesDir, theme);
		try {
			Logger.info("Loading device-theme: " + theme);
			deviceIcons = new Hashtable<String, ImageIcon>();
			for (File f : themeDir.listFiles()) {
				deviceIcons.put(f.getName().replace(".png", ""), new ImageIcon(f.getAbsolutePath()));
			}
			currentDeviceTheme = theme;
			Settings.put("device.theme", theme);
		} catch (Exception e) {
			Logger.fatal("Unable to load icons from "+themeDir.getAbsolutePath(), e);
		}
	}
	
	public static ImageIcon fromBase64(String s) {
		ImageIcon img = customCache.get(s);
		if (img == null) {
			try {
				img = new ImageIcon(Base64.decode(s));
				img.setDescription(s);
				customCache.put(s, img);
			} catch (IOException e) {
				Logger.error("Unable to read encoded image", e);
				img = getCisco("workstation");
			}
		}
		return img;
	}
	
	/**
	 * Extracts a Base64-String from a PNG-File
	 * @param f png-file
	 * @return base64
	 */
	public static String getBase64(File f) {
		if (f != null) {
			byte[] b = getPNGBytes(f);
			if (b != null) {
				return Base64.encodeBytes(b);
			}
		}
		return null;
	}
	
	private static byte[] getPNGBytes(File f) {
		ByteArrayOutputStream baos = new ByteArrayOutputStream(1000);
		try {
			BufferedImage orig = ImageIO.read(f);
			BufferedImage resized = new BufferedImage(64, 64, BufferedImage.TYPE_INT_ARGB);
				resized.getGraphics().drawImage(orig, 0, 0, 64, 64, null);
			ImageIO.write(resized, "png", baos);
			baos.flush();
			return baos.toByteArray();
		} catch (IOException e) {
			Logger.error("Unable to read image "+f, e);
			return null;
		} finally {
			try {
				baos.close();
			} catch (IOException e) {}			
		}
	}
	
	private Icons() {}
}
